#!/usr/bin/perl -w
use strict;
use sigtrap;

# Program copyright of Philipp L. Wesche, 2003

# This program is being released under the terms of the General Public License,
# to be found at
# http://www.gnu.org/copyleft


#TODO: using HTML::FormatText is effective but hugely inefficient!

#TODO: how to parse and print special issue entries?
#TODO: what about journals that appear more than once a month, or yearly?

my @files = `ls *.html`;
my $output = "";

for (my $a = 0; $a < @files; ++$a) {
	open IN, $files[$a];
	chomp (my @lines = <IN>);
	close IN;

	my $title;
	my $authors;
	my $journal;
	my $volume;
	my $pages;
	my $year;
	my $issue;
	my $date;
	my $quickref;

	for (my $linecount = 0; $linecount < @lines; ++$linecount) {
		if ($lines[$linecount] =~ /^ <center>/) {
			# get title
			chomp ($title = &convert_to_text(\$lines[$linecount]));
	
			# get authors
			chomp (my $authors_given = &convert_to_text(\$lines[$linecount + 1]));
			($authors, my $quickauthors) = &format_authors($authors_given);
	
			# get journal
			chomp ($journal = &convert_to_text(\$lines[$linecount + 2]));
			$journal = &decapitalise_journal($journal);
	
			# get issue etc.
			chomp (my $exact = &convert_to_text(\$lines[$linecount + 4]));
			($volume, $issue, $pages, $year, $date, my $quickyear) = &format_issue($exact);
#			print "$volume ($issue): $pages. $date, $year.\n";
			$quickref = $quickauthors.$quickyear;
		}
	}

	$output .= "\@article{$quickref,\n";
	$output .= "  author =\t  \"$authors\",\n";
	$output .= "  title =\t  \"$title\",\n";
	$output .= "  journal =\t \"$journal\",\n";
	$output .= "  year =\t  $year,\n";
	$output .= "  volume =\t  $volume,\n";
	$output .= "  pages =\t  \"$pages\",\n";
	# $issue and $date could also be used
	$output .= "}\n\n";

}

open (OUT, ">refs.bib");
print OUT $output;
close OUT;

exit;



sub convert_to_text {
	my $pointer = $_[0];
	my $html = $$pointer;
	
	require HTML::TreeBuilder;
	my $tree = HTML::TreeBuilder->new->parse($html);
	require HTML::FormatText;
	my $formatter = HTML::FormatText->new(leftmargin => 0, rightmargin => 1000);
	my $outlines = $formatter->format($tree);
	return ($outlines);
}




sub format_authors {
	my @authorlist = split(/, /, $_[0]);
	my $authors = "";
	my $quickauths = "";

	for ( my $i = 0; $i < @authorlist; ++$i) {
		# find initials and surname
		my @temp = split (/ /, $authorlist[$i]);
		my $given_initials = $temp[@temp - 1];
		my $surname = join(" ", @temp[0..(@temp - 2)]);

		# prepare author handle
		if (@authorlist == 1) {
			if ($i == 0) {
				$quickauths = $surname;
			}
		} elsif (@authorlist == 2) {
			if ($i < 2) {
				$quickauths.= $surname;
			}
		} elsif (@authorlist == 3) {
			if ($i < 3) {
				$quickauths .= $surname;
			}
		} else {
			if ($i == 0 or $i == (@authorlist - 1)) {
				$quickauths .= $surname;
			}
		}

		# format initials
		my @letters = split(//, $given_initials);
		my $initials = "";
		for (my $e = 0; $e < @letters; ++$e) {
			if ($e == (@letters - 1)) {
				$initials .= $letters[$e].".";
			} else {
				$initials .= $letters[$e].". ";
			}
		}

		# prepare output
		if ($i == @authorlist - 1) {
			$authors .= $surname.", ".$initials;
		} else {
			$authors .= $surname.", ".$initials." and ";
		}
	}

	$quickauths =~ tr/A-Z/a-z/;

	return ($authors, $quickauths);
}


sub format_issue {
	my @space_divided = split (/ /, $_[0]);
	my $year = $space_divided[@space_divided - 1];
	my $volume = $space_divided[0];
	my $pages = $space_divided[2];
	$pages =~ s/-/--/g;
	my $issue = $space_divided[1];
	$issue =~ s/\(|\)://g;
	my $date = $space_divided[@space_divided - 2];
	my $quickyear = join("", (split(//, $year))[2..3]);
	return ($volume, $issue, $pages, $year, $date, $quickyear);
}



sub decapitalise_journal {
	my @words = split (/ /, $_[0]);
	my $output;
	for (my $ei = 0; $ei < @words; ++$ei) {
		my $q = $words[$ei];
		if ($q eq "OF" || $q eq "IN" || $q eq "AND" || $q eq "THE" || $q  eq "FOR" || $q eq "ON") {
			$q =~ tr/A-Z/a-z/;
			$output .= $q." ";
		} elsif ($q eq '&') {
			$output .= "and ";
		} else {
			my @letters = split(//, $q);
			for (my $u = 1; $u < @letters; ++$u) {
				$letters[$u] =~ tr/A-Z/a-z/;
			}
			if ($ei == (@words - 1)) {
				$output .= join("", @letters);
			} else {
				$output .= join("", @letters)." ";
			}
		}
	}
	return $output;
}

