#!/usr/bin/perl -w
use strict;
use sigtrap;

# Program copyright Philipp L. Wesche 2003
# This program has been released under the terms of the General Public License,
# to be found at:
# http://www.gnu.org/copyleft

open IN, $ARGV[0];
chomp (my @lines = <IN>);
close IN;

my @pertlines;
my @refhandles;
my %sortme;

for (my $ct = 0; $ct < @lines; ++$ct) {
	if ($lines[$ct] =~ /^@/ && $lines[$ct] !~ /^@[Ss][Tt][Rr][Ii][Nn][Gg]/) {
		push (@pertlines, $ct);
		push (@refhandles, (split(/{|,/, $lines[$ct]))[1]);
	}
}

# remove any appended a,b,c etc.
for (my $little = 0; $little < @refhandles; ++$little) {
	if ($refhandles[$little] =~ /[a-z]$/) {
		$refhandles[$little] =~ s/[a-z]$//;
		$lines[$pertlines[$little]] =~ s/$refhandles[$little].?[a-z]/$refhandles[$little]/;
								# the .? avoids an ugly ambiguity in Perl
	}
}

(my $refpointer, my $linepointer) = &find_same(\@refhandles, \@lines, \@pertlines);
@refhandles = @$refpointer;
@lines = @$linepointer;


for (my $city = 0; $city < @pertlines; ++$city) {
	if ($city == @pertlines - 1) {
		$sortme{$refhandles[$city]} = join("\n", @lines[$pertlines[$city]..@lines - 1]);
	} else {
		$sortme{$refhandles[$city]} = join("\n", @lines[$pertlines[$city]..($pertlines[$city + 1] - 1)]);
	}
}

my @auxiliary = sort keys %sortme;

open OUT, ">$ARGV[0]";

foreach (@auxiliary) {
	print OUT "$sortme{$_}\n";
}

close OUT;
exit;



sub find_same {
	my $pointer = $_[0];
	my @refhandles = @$pointer;
	my $linepointer = $_[1];
	my @lines = @$linepointer;
	my $pertpoint = $_[2];
	my @perts = @$pertpoint;

	# this algorithm can be made more efficient by ignoring those @refhandles which have already been fixed
	# however, that gives you execution times 10% shorter for files with many different papers from the same author and year
	# BUT it can give you execution times 33% longer for more heterogeneous files, so I reckon it's not worth doing

	# another improvement worth considering:
	# once you've found several papers from same author/year,
	# they are already sorted, so you should pass them to the
	# sort function as one block, with only the first handle
	# taken into account in sorting
	# HOWEVER, since the sorting takes a negligible fraction of execution time,
	# you are likely to make matters worse! I tried and told you.

	for (my $oh = 0; $oh < @refhandles - 1; ++$oh) {

		# for a slight efficiency gain
		if ($refhandles[$oh] =~ /[a-z]$/) {
			next;
		}


		my @signore;
		push (@signore, $oh);
		my $switch = 0;

		for (my $ah = $oh + 1; $ah < @refhandles; ++$ah) {
			if ($refhandles[$oh] eq $refhandles[$ah]) {
				push (@signore, $ah);
			}
		}
		if (defined($signore[1])) {
			for (my $i = 0; $i < @signore; ++$i) {
				my $asciival = 97 + $i;
				my $char = chr($asciival);
				$lines[$perts[$signore[$i]]] =~ s/$refhandles[$signore[$i]]/$refhandles[$signore[$i]]$char/g;
				$refhandles[$signore[$i]] .= $char;
			}
		}
	}
	return (\@refhandles, \@lines);
}
